// -----------------------------------------------------------------------------
//
// Copyright 2011(c) Analog Devices, Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//  - Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  - Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//  - Neither the name of Analog Devices, Inc. nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//  - The use of this software may or may not infringe the patent rights
//    of one or more patent holders.  This license does not release you
//    from the requirement that you obtain separate licenses from these
//    patent holders to use this software.
//  - Use of the software either in source or binary form, must be run
//    on or directly connected to an Analog Devices Inc. component.
//
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// -----------------------------------------------------------------------------
// FILE NAME :      AD7689.v
// MODULE NAME :    AD7689
// AUTHOR : Adrian Costina
// AUTHORS EMAIL : Adrian.Costina@analog.com
// -----------------------------------------------------------------------------
// SVN REVISION: $WCREV$
// -----------------------------------------------------------------------------
// KEYWORDS : AD7689, EvalBoard
// -----------------------------------------------------------------------------
// PURPOSE : Module used to retrieve data from the AD7689
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Reset Strategy      : reset on '1'
// Clock Domains       : fpga_clk_i and adc_clk_i are 20MHz
// Critical Timing     :
// Test Features       :
// Asynchronous I/F    :
// Instantiations      :
// Synthesizable (y/n) : Y
// Target Device       : Cyclone II EP2C5F256C7N from AD7689 EvalBoard
// Other               :
// -----------------------------------------------------------------------------

`timescale 1 ns/ 100 ps

//------------------------------------------------------------------------------
//----------- Module Declaration -----------------------------------------------
//------------------------------------------------------------------------------
module AD7689 (
//----------- Ports Declarations -----------------------------------------------
// Clock and reset signals

        input               fpga_clk_i,         // System clock
        input               adc_clk_i,          // Clock to be applied to ADC to drive the conversions
        input               reset_i,            // Active high reset signal

// IP control and data interface
        input               wr_data_n_i,        // Active low signal to initiate a data write to the ADC
        input      [15:0]   data_i,             // Data to be written to the ADC

        output reg [127:0]  data_channels_o,    // Data read from the ADC
        output reg          data_rd_ready_o,    // When set to high the data read from the ADC is available on the data bus
        output reg          data_wr_ready_o,    // Used to signal the status of an ADC write: 0 - write in progress, 1 - write complete

// External ADC connections
        input               MISO_i,             // Master Input Slave Output
        output              MOSI_o,             // Master Output Slave Input
        output              SCLK_o,             // Serial Data Clock
        output reg          CNV_o               // Pin to control the Start of a new Conversion
);

//------------------------------------------------------------------------------
//----------- Local Parameters -------------------------------------------------
//------------------------------------------------------------------------------
// State Machine - State Names (parameters)
parameter IDLE              = 9'b000000001; // state in which a write can be performed to AD7689
parameter START_CNV         = 9'b000000010; // start the conversion process
parameter CONVERSION        = 9'b000000100; // wait conversion time
parameter CNV_LOW           = 9'b000001000; // bring CNV low
parameter READ_DATA         = 9'b000010000; // perform read data, before busy is activated
parameter SELECT_CHANNEL    = 9'b000100000; // depending on the configuration, selects the next state
parameter SYNCHRONIZE_COM   = 9'b001000000; // state in which samples are referenced to COM pin
parameter SYNCHRONIZE_DIFF  = 9'b010000000; // state in which samples are differential
parameter WAIT_CYCLE_END    = 9'b100000000; // waits for the conversion time to pass

localparam real      FPGA_CLK        = 20;  //MHz
localparam real      ADC_CYCLE_TIME  = 4;   // us
localparam real      ADC_CONV_TIME   = 2.2; // us

localparam integer    CYCLE_CNT       = (FPGA_CLK * ADC_CYCLE_TIME) - 3;    // to be used as cycle counter
localparam integer    CONV_CNT        = (FPGA_CLK * ADC_CONV_TIME) - 2 ;    // to be used as conversion counter

//------------------------------------------------------------------------------
//----------- Registers Declarations -------------------------------------------
//------------------------------------------------------------------------------
// State Machine Registers
reg [8:0]   present_state;  // Current State of the State Machine
reg [8:0]   next_state;     // Next State of the State Machine
reg         SCLK_en;        // Used to generate SCLK signal
reg [15:0]  tx_reg;         // Transmitted data, as received from the CED1Z_interface module
reg [15:0]  tx_reg_d;       // Register used for sending data
reg [15:0]  rx_reg;         // Register used for acquiring data
reg [31:0]  cycle_cnt;      // Register used for measuring the cycle time
reg [31:0]  conv_cnt;       // Register used for measuring the conversion time
reg [ 4:0]  sclk_cnt;       // Register used for counting the clock signals sent
reg [1:0]   seq;            // Register used for configuring the sequencer
reg [2:0]   inx;            // Register used for input channel selection
reg [2:0]   incc;           // Register used for input channel configuration
reg [2:0]   channel;        // Register used for keeping track of the channel acquired

//------------------------------------------------------------------------------
//----------- Assign/Always Blocks ---------------------------------------------
//------------------------------------------------------------------------------
assign SCLK_o           = SCLK_en ? adc_clk_i : 1'b0;   // clock is idle LOW
assign MOSI_o           = tx_reg_d[13];                 // send data from tx_reg_d register on the MOSI data line

always @(negedge adc_clk_i)                             // Process for transmitting/receiving data to/from AD7689
begin
    if (reset_i == 1'b1)
    begin
        rx_reg      <= 16'h0;
        tx_reg_d    <= 16'h0;
    end
    else if(SCLK_en == 1'b1)
    begin
        rx_reg      <= { rx_reg[14:0], MISO_i };        // Receive data
        tx_reg_d    <= { tx_reg_d[14:0], 1'b0 };        // Send data
    end
    else if (present_state == IDLE)                     // clear rx_reg register
    begin
        rx_reg      <= 16'h0;
    end
    else if (present_state == CONVERSION)               //update tx_reg_d register
    begin
        tx_reg_d    <= {1'b0, tx_reg[13], tx_reg[13:0]};
    end
end

always @(negedge fpga_clk_i)            // Process for timing the conversion time
begin
    if (present_state == START_CNV)     // Reset counters in state START_CNV
    begin
        cycle_cnt   <= CYCLE_CNT;
        conv_cnt    <= CONV_CNT;
    end
    else
    begin
        if ( cycle_cnt != 0 )           // Decrease cycle counter
        begin
            cycle_cnt   <= cycle_cnt - 32'h1;
        end
        if ( conv_cnt != 0 )            // Decrease conversion counter
        begin
            conv_cnt    <= conv_cnt - 32'h1;
        end
    end
end

// State Machine - Select State
always @( present_state, sclk_cnt, cycle_cnt, incc, conv_cnt )
begin

    next_state <= present_state;
    case(present_state)
        IDLE:
        begin
            next_state      <= START_CNV;
        end
        START_CNV:
        begin
            next_state      <= CONVERSION;
        end
        CONVERSION:
        begin
            if(conv_cnt == 32'b0)
            begin
                next_state  <= CNV_LOW;
            end
        end
        CNV_LOW:
        begin
            next_state      <= READ_DATA;
        end
        READ_DATA:
        begin
            if(sclk_cnt == 5'b0)
            begin
                next_state  <= SELECT_CHANNEL;
            end
        end
        SELECT_CHANNEL:
        begin
            if( incc == 3'b000 || incc == 3'b001 || incc == 3'b100 || incc == 3'b101)   // Configuration words with differential pairs
            begin
                next_state  <= SYNCHRONIZE_DIFF;
            end
            else
            begin
                next_state  <= SYNCHRONIZE_COM;
            end
        end
        SYNCHRONIZE_COM:
        begin
            next_state  <= WAIT_CYCLE_END;
        end
        SYNCHRONIZE_DIFF:
        begin
            next_state  <= WAIT_CYCLE_END;
        end
        WAIT_CYCLE_END:
        begin
            if (cycle_cnt == 32'b0)
            begin
                next_state  <= IDLE;
            end
        end
        default:
        begin
            next_state      <= IDLE;
        end
    endcase
end

// State Machine - Select Output
always @(negedge fpga_clk_i)
begin
    if(reset_i == 1'b1)
    begin
        present_state               <= IDLE;
        tx_reg                      <= 16'h0000;
        data_channels_o [ 127:0]    <= 128'h0;
    end
    else
    begin
        present_state           <= next_state;
        case( present_state )
            IDLE:
            begin
                if (wr_data_n_i == 0)   // if a write has been performed from  CED1Z to ADCCFG register, update the configuration register on AD7689
                begin
                    tx_reg                      <= data_i;
                    data_wr_ready_o             <= 1'b1;
                    channel                     <= 3'h5;
                    incc                        <= data_i[12:10];
                    inx                         <= data_i[9:7];
                    seq                         <= data_i[2:1];
                    data_channels_o [ 127:0]    <= 128'h0;
                end
                else
                begin                   // if no write has been performed, keep DIN low
                    tx_reg      <= 16'h0;
                end
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
                sclk_cnt        <= 5'hf;
            end
            START_CNV:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b1;
                data_rd_ready_o <= 1'b0;
            end
            CONVERSION:                 // The CNV pin is kept high until EOC. Configuration without busy indicator
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b1;
                data_rd_ready_o <= 1'b0;
            end
            CNV_LOW:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
            end
            READ_DATA:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b1;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
                if( sclk_cnt != 0 )
                begin
                    sclk_cnt    <= sclk_cnt - 5'h1;
                end
            end
            SELECT_CHANNEL:
            begin
                channel         <= channel + 3'h1;
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                data_rd_ready_o <= 1'b0;
                CNV_o           <= 1'b0;
            end
            SYNCHRONIZE_COM: // Using 11 for channel sequencer on and 00 for channel sequencer disabled. 01 and 10 should not be used to configure the AD7689
            begin
                case( channel )
                    3'h0:
                    begin
                        data_channels_o[127:112] <= rx_reg;
                        if ( seq == 2'b11  && inx == 3'h0)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h1:
                    begin
                        data_channels_o[111:96] <= rx_reg;
                        if ( seq == 2'b11 && inx == 3'h1)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h2:
                    begin
                        data_channels_o[95:80]  <= rx_reg;
                        if (seq == 2'b11 && inx == 3'h2)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h3:
                    begin
                        data_channels_o[79:64]  <= rx_reg;
                        if (seq == 2'b11 && inx == 3'h3)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h4:
                    begin
                        data_channels_o[63:48]  <= rx_reg;
                        if (seq == 2'b11 && inx == 3'h4)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h5:
                    begin
                        data_channels_o[47:32]  <= rx_reg;
                        if (seq == 2'b11 && inx == 3'h5)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h6:
                    begin
                        data_channels_o[31:16]  <= rx_reg;
                        if (seq == 2'b11 && inx == 3'h6)
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    default:
                    begin
                        data_channels_o[15:0]   <= rx_reg;
                        data_rd_ready_o         <= 1'b1;
                        channel                 <= 3'h7;
                    end
                endcase
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
            end
            SYNCHRONIZE_DIFF:
            begin
                case( channel )
                    3'h0:
                    begin
                        data_channels_o[127:112] <= rx_reg;
                        if (seq == 2'b11 && (inx == 3'h0 || inx == 3'h1))
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h1:
                    begin
                        data_channels_o[111:96]  <= rx_reg;
                        if ( seq == 2'b11 && (inx == 3'h2 || inx == 3'h3))
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h2:
                    begin
                        data_channels_o[95:80]  <= rx_reg;
                        if (seq == 2'b11 && (inx == 3'h4 || inx == 3'h5))
                        begin
                            data_rd_ready_o <= 1'b1;
                            channel         <= 3'h7;
                        end
                    end
                    3'h6:
                    begin
                        channel             <= 3'h6; //used to synchronnize channels after configuration change
                    end
                    default:
                    begin
                        data_channels_o[79:64]  <= rx_reg;
                        data_rd_ready_o         <= 1'b1;
                        channel                 <= 3'h7;
                    end
                endcase
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
            end
            WAIT_CYCLE_END:
            begin
                data_wr_ready_o <= 1'b0;
                SCLK_en         <= 1'b0;
                CNV_o           <= 1'b0;
                data_rd_ready_o <= 1'b0;
            end
        endcase
    end
end

endmodule
